package cn.bmob.data_plugin;

import android.content.Context;
import android.widget.Toast;

import cn.bmob.data_plugin.util.InstallationUtils;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/**
 * DataPlugin
 */
public class DataPlugin implements MethodCallHandler {

    /**
     * TODO 1、新增、删除或修改原生方法的时候，需要重启应用才能使得flutter正常调用原生方法，否则会因为缓存问题导致调用失败。
     */
    private Context mContext;
    private static final String GET_INSTALLATION_ID = "getInstallationId";
    private static final String GET_PLATFORM_VERSION = "getPlatformVersion";
    private static final String TOAST = "showToast";
    private static String CHANNEL = "data_plugin";

    public DataPlugin(Registrar registrar) {
        mContext = registrar.context();
    }


    /**
     * Plugin registration.
     */
    public static void registerWith(Registrar registrar) {
        final MethodChannel channel = new MethodChannel(registrar.messenger(), CHANNEL);
        channel.setMethodCallHandler(new DataPlugin(registrar));
    }

    @Override
    public void onMethodCall(MethodCall call, Result result) {
        String method = call.method;
        switch (method) {
            //获取平台版本
            case GET_PLATFORM_VERSION:
                result.success(android.os.Build.VERSION.RELEASE);
                break;
            //获取installation id
            case GET_INSTALLATION_ID:
                String installationId = InstallationUtils.getInstallationId(mContext);
                result.success(installationId);
                break;
            //弹出 toast
            case TOAST:
                String msg = call.argument("msg");
                Toast.makeText(mContext, msg, Toast.LENGTH_SHORT).show();
                break;
            default:
                result.notImplemented();
                break;
        }
    }
}
